import boto3


class SES:
    def __init__(self, boto3_session, region_name='us-east-1'):
        self.client = boto3_session.client('ses', region_name=region_name)
        self.region = boto3_session.region_name

    def send_email(self, sender, recipient, subject, body_text, body_html):
        # SES 클라이언트 생성
        ses_client = self.client
        response = ses_client.send_email(
            Destination={
                'ToAddresses': [recipient],
            },
            Message={
                'Body': {
                    'Html': {
                        'Charset': "UTF-8",
                        'Data': body_html,
                    },
                    'Text': {
                        'Charset': "UTF-8",
                        'Data': body_text,
                    },
                },
                'Subject': {
                    'Charset': "UTF-8",
                    'Data': subject,
                },
            },
            Source=sender,
        )
        return response



"""
This is sample code to tell how to use sawsi framework.
You could delete this file after know how to use this framework.
"""

import api
from {{app}}.model.sample_user import User


def get_user_by_session(session_id:str)->User:
    address_data = {'street': '123 Main St', 'city': 'Anytown', 'country': 'USA'}
    user_data = {'name': 'John Doe', 'age': 30, 'gender': 'male', 'address': address_data}
    user = User(**user_data)
    return user



import time

from sawsi.shared import error_util
from sawsi.shared import handler_util
import errs


# 아래 핸들러는 share.error_util.AppError 에러가 발생할시에, 자동으로
# 에러를 response 객체에 담아 코드와 메시지로 구분하여 전송합니다.
@handler_util.aws_handler_wrapper(
    error_receiver=lambda errmsg: print(errmsg),  # 이 Lambda 함수를 슬랙 Webhook 등으로 대체하면 에러 발생시 모니터링이 가능합니다.
    content_type='application/json',  # 기본적으로 JSON 타입을 반환합니다.
    use_traceback=True,  # 에러 발생시 상세 값을 응답에 전달할지 유무입니다.
    ignore_app_errors=[
        errs.no_session
    ]
)
def handler(event, context):
    """
    AWS LAMBDA에서 API Gateway 를 통해 콜한 경우
    """
    # API Gateway 로부터 Lambda 에 요청이 들어오면 다음과 같이 body 와 headers 를 분리하여 dict 형태로 반환합니다.
    body = handler_util.get_body(event, context)
    headers = handler_util.get_headers(event, context)
    source_ip = handler_util.get_source_ip(event, context)

    # 아래부터는 사용자가 직접 응용하여 핸들러를 구성, 다른 함수들로 라우팅합니다.
    cmd = body.get('cmd', None)

    # 로그인을 수행하는 API
    if cmd == '{{app}}.controller.sample_login.login':
        import {{app}}.controller.sample_login
        return {{app}}.controller.sample_login.login(
            body['email'], body['password']
        )
    # ... 세션 할당 이전 구현할 수 있는 API 들을 구현


    # 헤더로부터 세션 가져오기
    session_id = headers.get('session_id', None)
    if not session_id:
        raise errs.no_session

    # 아래부터는 세션이 있어야 활용 가능한 API 구현

    # 자기 자신의 정보를 세션으로 가져오는 API
    if cmd == '{{app}}.controller.sample_login.get_me':
        import {{app}}.controller.sample_login
        return {{app}}.controller.sample_login.get_me(
            session_id
        )

    # 핸들러 CMD 에 해당하는 CMD 값이 없을 경우 에러 발생
    raise error_util.SYSTEM_NO_SUCH_CMD



"""
빌드 준비에 필요한 코드 호출
샘플입니다. 적절히 변경해서 사용해주세요.
"""


import api
import config
import {{app}}.doc_make.test_and_make_api_doc
from {{app}}.model.sample_user import Address, User


if __name__ == '__main__':
    config.build = True
    # DB 초기화 등 진행.
    Address.sync_table()
    User.sync_table()
    api.locking.init_table()
    api.firehose_log.init()
    api.s3_public.init_s3_bucket()
    # 초기화 메서드를 제공하지 않는 리소스는, AWS Console 에서 직접 생성 요망

    if config.env == 'dev':
        {{app}}.doc_make.test_and_make_api_doc.test_and_make_api_doc()



from pydantic import BaseModel
from typing import TypeVar, Generic, Type
from typing import Literal, get_origin, get_args, Union


# 현재 클래스 형을 나타내는 TypeVar 생성
T = TypeVar('T', bound='DynamoModel')

class DynamoModel(BaseModel, Generic[T]):
    @classmethod
    def _table_name(cls):
        return cls._table.get_default()

    @classmethod
    def get_item(cls: Type[T], id: str, consistent_read: bool = True) -> T:
        data = cls.__dynamo__.get_item(
            cls._table_name(), id, consistent_read
        )
        return cls(**data)

    @classmethod
    def sync_table(cls):
        """
        테이블이 없으면 생성, 인덱스도 없으면 생성합니다.
        :return:
        """
        cls.__dynamo__.create_table(cls._table_name())
        for partition_key, sort_key in cls._indexes.get_default():
            partition_key_type = extract_dynamodb_type(cls.model_fields[partition_key].annotation)
            if sort_key:
                sort_key_type = extract_dynamodb_type(cls.model_fields[sort_key].annotation)
                cls.__dynamo__.create_global_index(
                    cls._table_name(),
                    partition_key, partition_key_type,
                    sort_key, sort_key_type
                )
            else:
                cls.__dynamo__.create_global_index(
                    cls._table_name(),
                    partition_key, partition_key_type,
                )


def extract_dynamodb_type(annotation) -> str:
    # Optional 타입 처리
    if get_origin(annotation) is Union:
        args = get_args(annotation)
        # Optional[X]는 Union[X, NoneType]과 같으므로, NoneType을 제외하고 처리
        non_none_args = [arg for arg in args if arg is not type(None)]
        if len(non_none_args) == 1:
            return extract_dynamodb_type(non_none_args[0])

    # Literal 타입 처리
    if get_origin(annotation) is Literal:
        # Literal 내부의 모든 값이 문자열이면 'S', 그렇지 않으면 예외 처리
        args = get_args(annotation)
        if all(isinstance(arg, str) for arg in args):
            return 'S'
        else:
            raise ValueError("DynamoDB does not support this kind of Literal directly.")

    # 기본 타입 처리
    if annotation is int or annotation is float:
        return 'N'
    elif annotation is str:
        return 'S'
    elif annotation is bytes:
        return 'B'

    # 지원되지 않는 타입
    raise ValueError(f"Unsupported type annotation: {annotation}")
